<?php

/**
 * @file
 * Verify IPN updates from BitPay and update relevant transactions.
 */

/**
 * Interpret IPN updates from BitPay and update relevant payment transactions.
 */
function commerce_bitpay_verify_ipn() {
  $payment_method = commerce_payment_method_instance_load('bitpay|commerce_payment_bitpay');
  $payment_method['settings'] += commerce_bitpay_default_settings();

  libraries_load('php-client');

  $json = bpVerifyNotification($payment_method['settings']['apiKey']);

  if (is_null($json) || is_string($json) || !isset($json['posData']) || !is_numeric($json['posData'])) {
    switch ($json) {
      case 'No post data':
        watchdog('commerce_bitpay', 'IPN POST had no data.', array(), WATCHDOG_WARNING);
        break;

      case 'no posData':
        watchdog('commerce_bitpay', 'No IPN posData value was provided.', array(), WATCHDOG_WARNING);
        break;

      case 'authentication failed (bad hash)':
        watchdog('commerce_bitpay', 'IPN verification failed (bad hash).', array(), WATCHDOG_WARNING);
        break;

      default:
        watchdog('commerce_bitpay', 'IPN posted something strange: @string', array('@string' => $json));
    }
    return FALSE;
  }

  $order_id = db_query("SELECT order_id FROM {commerce_payment_transaction} WHERE transaction_id=:id", array(':id' => $json['posData']))->fetchField();
  if (empty($order_id) || !is_numeric($order_id)) {
    watchdog('commerce_bitpay', 'Failed to retrieve an order ID for IPN.');
    return FALSE;
  }
  watchdog('commerce_bitpay', 'IPN received for order @order_id, transaction @trans_id.', array('@trans_id' => $json['posData'], '@order_id' => $order_id), WATCHDOG_DEBUG, l(t('view order payments'), 'admin/commerce/orders/' . $order_id . '/payment'));

  $transaction = commerce_payment_transaction_load($json['posData']);
  $order = commerce_order_load($order_id);
  // BitPay doesn't appear to send an IPN at all unless some action is taken
  // towards paying the invoice. This assumes some sort of payment attempt.
  commerce_checkout_complete($order);

  switch ($json['status']) {
    case 'new':
      $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
      $transaction->message = t("Waiting for payment.");
      $transaction->remote_status = $json['status'];
      break;

    case 'paid':
      $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
      $transaction->message = t('Paid, waiting for blockchain confirmation.');
      $transaction->remote_status = $json['status'];
      break;

    case 'confirmed':
      $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $transaction->message = t('Payment confirmed.');
      $transaction->remote_status = $json['status'];
      break;

    case 'complete':
      $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $transaction->message = t('Merchant account credited. Invoice complete.');
      $transaction->remote_status = $json['status'];
      break;

    case 'expired':
      // From testing, expired invoices don't actually generate an IPN reply,
      // but this is here just in case some condition does actually trigger it.
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = t('Payment was not received within 15 minutes from invoice generation on transaction @txn_id.', array('@txn_id' => $json['posData']));
      $transaction->remote_status = $json['status'];
      break;

    case 'invalid':
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = t('Payment was received but was not confirmed within 1 hour for transaction @txn_id.', array('@txn_id' => $json['posData']));
      $transaction->remote_status = $json['status'];
      break;

    default:
      watchdog('commerce_bitpay', 'Unknown payment status for transaction @id: @status', array('@id' => $json['posData'], '@status' => $json['status']), WATCHDOG_WARNING);
  }

  commerce_payment_transaction_save($transaction);
}
